@@ -79,16 +79,17 @@ cv.georob <-
   ## 2016-07-20 AP changes for parallel computations
   ## 2016-08-17 AP changes for nested variogram models
   ## 2016-08-22 AP correcting error when computing  standard errors of cv kriging predictions
+  ## 2018-01-05 AP improved memory management in parallel computations
+  ##							 improved error handling during parallel computations
 
 
   ## auxiliary function that fits the model and computes the predictions of
   ## a cross-validation set
 
-  f.aux <- function(
-    ..i.., object, formula, data, sets,
-    variogram.object, is.mat.param.aniso, lgn,
-    verbose, ...
-  ){  ## cv function
+  f.aux <- function( ..i..,  ... ){  ## cv function
+    
+    ## object, formula, data, sets, variogram.object, is.mat.param.aniso,
+    ## lgn, verbose are taken from parent enviroment
 
     if (verbose) cat( "\n\n  processing cross-validation set", ..i.., "\n" )
 
@@ -627,35 +628,43 @@ cv.georob <-
   object[["call"]] <- cl
 
   ## loop over all cross-validation sets
+  
+  ## set default value for control of forking if missing (required for backward compatibility)
+  
+  if( is.null( object[["control"]][["pcmp"]][["fork"]] ) ){
+    object[["control"]][["pcmp"]][["fork"]] <- !identical( .Platform[["OS.type"]], "windows" )
+  }
 
-  if( ncores > 1L && .Platform[["OS.type"]] == "windows" ){
+  if( ncores > 1L && !object[["control"]][["pcmp"]][["fork"]] ){
 
     ## create a SNOW cluster on windows OS
 
-    cl <- makePSOCKcluster( ncores, outfile = "")
+    clstr <- makeCluster( ncores, type = "SOCK" )
+    save( clstr, file = "SOCKcluster.RData" )
+    options( error = f.stop.cluster )
 
     ## export required items to workers
 
-    junk <- clusterEvalQ( cl, require( georob, quietly = TRUE ) )
-
+    junk <- clusterEvalQ( clstr, require( georob, quietly = TRUE ) )
+    junk <- clusterExport( 
+      clstr,
+      c( 
+        "object", "formula", "data", "sets", "variogram.object", 
+        "is.mat.param.aniso", "lgn", "verbose"
+      ),
+      envir =  environment() 
+    )
+    
     t.result <- try(
       parLapply(
-        cl,
+        clstr,
         1L:length( sets ),
         f.aux,
-        object = object,
-        formula = formula,
-        data = data,
-        sets = sets,
-        variogram.object = variogram.object,
-        is.mat.param.aniso = is.mat.param.aniso,
-        lgn = lgn,
-        verbose = verbose,
         ...
       ), silent = TRUE
     )
 
-    stopCluster(cl)
+    f.stop.cluster( clstr )
 
   } else {
 
@@ -665,14 +674,6 @@ cv.georob <-
       mclapply(
         1L:length( sets ),
         f.aux,
-        object = object,
-        formula = formula,
-        data = data,
-        sets = sets,
-        variogram.object = variogram.object,
-        is.mat.param.aniso = is.mat.param.aniso,
-        lgn = lgn,
-        verbose = verbose,
         mc.cores = ncores,
         mc.allow.recursive = object[["control"]][["pcmp"]][["allow.recursive"]],
         ...
